////////////////////////////// 
// 複素数クラスの定義
////////////////////////////// 
var Complex = function( real, imag ){
	this.r = real || 0;
	this.i = imag || 0;
}
Complex.I = new Complex(0, 1);


Complex.prototype.set = function( real, imag ){
	this.r = real;
	this.i = imag;
	return this;
}
Complex.prototype.multiplyScalar = function( s ){
	this.r *= s;
	this.i *= s;
	return this;
}
Complex.prototype.add = function( z ){
	this.r += z.r;
	this.i += z.i;
	return this;
}
Complex.prototype.addComplexes = function( a, b ){
	this.r = a.r + b.r;
	this.i = a.i + b.i;
	return this;
}
Complex.prototype.sub = function( z ){
	this.r -= z.r;
	this.i -= z.i;
	return this;
}
Complex.prototype.subComplexes = function( a, b ){
	this.r = a.r - b.r;
	this.i = a.i - b.i;
	return this;
}
Complex.prototype.multiply = function( z ){
	this.multiplyComplexes(this, z);
	return this;
}
Complex.prototype.multiplyComplexes = function( a, b ){
	this.r = a.r * b.r - a.i * b.i;
	this.i = a.r * b.i + a.i * b.r;
	return this;
}

Complex.prototype.divide = function( z ){
	var absSq = z.r * z.r + z.i * z.i;
	this.r = ( this.r * z.r + this.i * z.i) / absSq;
	this.i = ( this.i * z.r - this.r * z.i) / absSq;
	return this;
}
Complex.prototype.divideComplexes = function( a, b ){
	var absSq = b.r * b.r + b.i * b.i;
	this.r = ( a.r * b.r + a.i * b.i) / absSq;
	this.i = ( a.i * b.r - a.r * b.i) / absSq;
	return this;
}
Complex.prototype.absSq = function( ){

	return this.r * this.r + this.i * this.i;
}
Complex.prototype.abs = function( ){

	return Math.sqrt( this.absSq() );
}

Complex.prototype.setExp = function( z ){
	this.r = Math.exp( z.r ) * Math.cos( z.i );
	this.i = Math.exp( z.r ) * Math.sin( z.i );	
	return this;
}

Complex.prototype.arg = function( ){

	if( this.r > 0 ) return Math.atan( this.i / this.r );
	else if( this.r == 0 && this.i > 0 ) return Math.PI /2;
	else if( this.r == 0 && this.i < 0 ) return 3 * Math.PI /2;
	else if( this.r < 0 ) return Math.atan( this.i / this.r ) + Math.PI;
	else return 0;

}

Complex.prototype.clone = function( ){

	return this.constructor().set( this.r, this.i );
}
Complex.prototype.copy = function( z ){
	this.r = z.r;
	this.i = z.i;
	return this;
}

////////////////////////////// 
// 複素数を要素とする２×２行列クラスの定義
////////////////////////////// 
var Matrix2c = function(){
	this.elements = [
		new Complex(1,0), new Complex(0,0),
		new Complex(0,0), new Complex(1,0)
	]
}

Matrix2c.prototype.set = function ( z11, z12, z21, z22 ) {

	var te = this.elements;

	te[ 0 ].copy( z11 ); te[ 1 ].copy( z12 ); 
	te[ 2 ].copy( z21 ); te[ 3 ].copy( z22 ); 

	return this;

}

Matrix2c.prototype.multiply = function( m ){

	this.multiplyMatrices( this, m );

	return this;

}
Matrix2c.prototype.multiplyMatrices = function( a, b ){

	var ae = a.elements;
	var be = b.elements;
	var te = this.elements;

	var a11 = ae[ 0 ], a12 = ae[ 1 ];
	var a21 = ae[ 2 ], a22 = ae[ 3 ];

	var b11 = be[ 0 ], b12 = be[ 1 ];
	var b21 = be[ 2 ], b22 = be[ 3 ];

	te[ 0 ] = new Complex().multiplyComplexes( a11, b11 ).add ( new Complex().multiplyComplexes( a12, b21 ) );
	te[ 1 ] = new Complex().multiplyComplexes( a11, b12 ).add ( new Complex().multiplyComplexes( a12, b22 ) );
	te[ 2 ] = new Complex().multiplyComplexes( a21, b11 ).add ( new Complex().multiplyComplexes( a22, b21 ) );
	te[ 3 ] = new Complex().multiplyComplexes( a21, b12 ).add ( new Complex().multiplyComplexes( a22, b22 ) );

	return this;
}

Matrix2c.prototype.determinant = function( ){

	var te = this.elements;

	var c1 = new Complex().multiplyComplexes(te[0], te[3]);
	var c2 = new Complex().multiplyComplexes(te[1], te[2]);

	return new Complex().subComplexes( c1, c2 );

}
